package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.dto.PromoterAndAgentDto;
import com.jpsoft.picc.modules.base.entity.*;
import com.jpsoft.picc.modules.base.service.*;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.service.ApplicationPolicyService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyMemberService;
import com.jpsoft.picc.modules.business.service.InsurancePolicyService;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.utils.NumberToCN;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import com.jpsoft.picc.modules.common.utils.VelocityHelper;
import com.jpsoft.picc.modules.sys.entity.DataDictionary;
import com.jpsoft.picc.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Months;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.xml.crypto.Data;
import java.math.BigDecimal;
import java.util.*;

@RestController
@RequestMapping("/insuranceApplication")
@Api(description = "投保单")
public class InsuranceApplicationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private JobsService jobsService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private JpCloudUserService jpCloudUserService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceApplication> create(){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        InsuranceApplication insuranceApplication = new InsuranceApplication();

        msgResult.setData(insuranceApplication);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceApplication> add(@RequestBody InsuranceApplication insuranceApplication,@RequestAttribute String subject){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            insuranceApplication.setId(UUID.randomUUID().toString());
            insuranceApplication.setDelFlag(false);
            insuranceApplication.setCreateBy(subject);
            insuranceApplication.setCreateTime(new Date());
            
            int affectCount = insuranceApplicationService.insert(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceApplication> edit(@PathVariable("id") String id){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            if (insuranceApplication != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取投保信息")
    @PostMapping("insureDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保信息ID",required = true,paramType = "query")
    })
    public MessageResult<InsuranceApplication> insureDetail(String applicationId){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            if (insuranceApplication != null) {
                InsurancePolicy insurancePolicy = applicationPolicyService.findFirstPolicyByApplicationId(applicationId);

                //首月投保
                insuranceApplication.setInsurancePolicy(insurancePolicy);

                //营业性质
//                if (StringUtils.isNotEmpty(insuranceApplication.getBusinessNature())){
//                    String businessNatureName = dataDictionaryService.findNameByCatalogNameAndValue("营业性质",insuranceApplication.getBusinessNature());
//                    insuranceApplication.setBusinessNatureN(businessNatureName);
//                }

                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceApplication> update(@RequestBody InsuranceApplication insuranceApplication,@RequestAttribute String subject){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            insuranceApplication.setUpdateBy(subject);
            insuranceApplication.setUpdateTime(new Date());
            
            int affectCount = insuranceApplicationService.update(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);
            insuranceApplication.setDelFlag(true);
            insuranceApplication.setUpdateBy(subject);
            insuranceApplication.setUpdateTime(new Date());

            int affectCount = insuranceApplicationService.update(insuranceApplication);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);
                insuranceApplication.setDelFlag(true);
                insuranceApplication.setUpdateBy(subject);
                insuranceApplication.setUpdateTime(new Date());

                affectCount += insuranceApplicationService.update(insuranceApplication);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="status",value = "状态",paramType = "query"),
            @ApiImplicitParam(name="definitionId",value = "险种ID",paramType = "query"),
            @ApiImplicitParam(name="companyName",value = "企业名称",paramType = "query"),
            @ApiImplicitParam(name="insuredName",value = "投保人姓名",paramType = "query"),
            @ApiImplicitParam(name="startTime",value = "开始时间",paramType = "query"),
            @ApiImplicitParam(name="endTime",value = "结束时间",paramType = "query"),
            @ApiImplicitParam(name="userType",value = "用户类型（0为人才超市，1为PICC）",paramType = "query"),
            @ApiImplicitParam(name="policyNo",value = "保险服务单号",paramType = "query")
    })
    public MessageResult<Map> pageList(
            String status,String definitionId,
            String insuredName,String companyName,
            String startTime,String endTime,String userType,String policyNo,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        if (StringUtils.isNotEmpty(definitionId)) {
            searchParams.put("definitionId",definitionId);
        }

        if (StringUtils.isNotEmpty(companyName)) {
            searchParams.put("companyName","%" + companyName + "%");
        }

        if (StringUtils.isNotEmpty(insuredName)) {
            searchParams.put("insuredName","%"+insuredName+"%");
        }

        if (StringUtils.isNotEmpty(startTime)) {
            searchParams.put("startTime",startTime);
        }

        if (StringUtils.isNotEmpty(endTime)) {
            searchParams.put("endTime",endTime);
        }

        if (StringUtils.isNotEmpty(userType)) {
            if(userType.equals("1")){
                searchParams.put("userType",userType);
            }
        }

        if (StringUtils.isNotEmpty(policyNo)) {
            searchParams.put("policyNo","%" + policyNo + "%");
        }

        Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="标签页数量提示")
    @RequestMapping(value = "tabCount",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="definitionId",value = "险种ID",paramType = "query"),
            @ApiImplicitParam(name="companyName",value = "企业名称",paramType = "query"),
            @ApiImplicitParam(name="insuredName",value = "投保人姓名",paramType = "query"),
            @ApiImplicitParam(name="startTime",value = "开始时间",paramType = "query"),
            @ApiImplicitParam(name="endTime",value = "结束时间",paramType = "query"),
            @ApiImplicitParam(name="userType",value = "用户类型（0为人才超市，1为PICC）",paramType = "query"),
            @ApiImplicitParam(name="policyNo",value = "保险服务单号",paramType = "query")
    })
    public MessageResult<Map> tabCount(
            String status,String definitionId,
            String insuredName,String companyName,
            String startTime,String endTime,String userType,String policyNo,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(definitionId)) {
            searchParams.put("definitionId",definitionId);
        }

        if (StringUtils.isNotEmpty(companyName)) {
            searchParams.put("companyName","%" + companyName + "%");
        }

        if (StringUtils.isNotEmpty(insuredName)) {
            searchParams.put("insuredName","%"+insuredName+"%");
        }

        if (StringUtils.isNotEmpty(startTime)) {
            searchParams.put("startTime",startTime);
        }

        if (StringUtils.isNotEmpty(endTime)) {
            searchParams.put("endTime",endTime);
        }

        if (StringUtils.isNotEmpty(userType)) {
            if(userType.equals("1")){
                searchParams.put("userType",userType);
            }
        }

        if (StringUtils.isNotEmpty(policyNo)) {
            searchParams.put("policyNo","%" + policyNo + "%");
        }

        Map<String,Object> countMap = new HashMap<>();

        Page<InsuranceApplication> allPage = insuranceApplicationService.pageSearch(searchParams,1,1,sortList);

        countMap.put("all",allPage.getTotal());

        for (PolicyStatus policyStatus : PolicyStatus.values()) {
            searchParams.put("status",policyStatus.getValue());
            Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams,1,1,sortList);
            countMap.put(policyStatus.getValue() + "" , page.getTotal());
        }

        msgResult.setResult(true);
        msgResult.setData(countMap);

        return msgResult;
    }

    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> acceptDetail(String policyId){

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> map = new HashMap<String,Object>();

        //投保单
        InsuranceApplication insuranceApplication = null;

        List<InsuranceApplication> appList = applicationPolicyService.findApplicationByPolicyId(policyId);

        if(appList.size()>0){
            insuranceApplication = appList.get(0);
        }

        String definitionId = insuranceApplication.getDefinitionId();

        InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(definitionId);

        map.put("id",insuranceDefinition.getId());
        map.put("name",insuranceDefinition.getName());

        List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(definitionId);

        List<Map<String,Object>> mapList = new ArrayList<Map<String,Object>>();

        Map<String,Object> varsMap = new HashMap<>();
        varsMap.put("num",insuranceApplication.getInsuredNumber());

        for(InsuranceDefinitionLimit  insuranceDefinitionLimit:insuranceDefinitionLimitList){
            Map<String,Object> map1 = new HashMap<String,Object>();
            map1.put("id",insuranceDefinitionLimit.getId());
            map1.put("name",insuranceDefinitionLimit.getName());

            //限额这里可能是公式，需要带入人数
            String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
            map1.put("limit",limit);

            String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
            map1.put("unit",unit);

            mapList.add(map1);
        }

        map.put("insuranceDefinitionLimitList",mapList);

        List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(insuranceApplication.getDefinitionId());

        List<Map<String,Object>> mapList1 = new ArrayList<Map<String,Object>>();

        DateTime endTime = new DateTime(insuranceApplication.getEndTime());

        DateTime startTime = new DateTime(insuranceApplication.getStartTime());

        //投保总月份
        int months = Months.monthsBetween(startTime, endTime).getMonths();

        BigDecimal totalAmount = new BigDecimal(0);

        for (InsuranceJobs insuranceJobs:InsuranceJobsList) {
            Map<String,Object> map1 = new HashMap<String,Object>();
            String jobId = insuranceJobs.getJobsId();
            map1.put("id",jobId);
            map1.put("name",insuranceJobs.getJobs().getName());
            map1.put("content",insuranceJobs.getJobs().getContent());
            map1.put("charges",insuranceJobs.getCharges());
            map1.put("chargesY",insuranceJobs.getCharges().multiply(new BigDecimal(12)));
            long memberCount = insurancePolicyMemberService.countByPolicyIdAndJobsId(policyId,insuranceJobs.getJobsId());
            long number = Long.valueOf(memberCount).intValue();
            map1.put("number",number);
            BigDecimal money = insuranceJobs.getCharges().multiply(new BigDecimal(months)).multiply(new BigDecimal(number));
            map1.put("money",money);

            totalAmount = totalAmount.add(money);
            mapList1.add(map1);
        }

        map.put("totalAmount",totalAmount);

        String cnMontrayUnit = NumberToCN.number2CNMontrayUnit(totalAmount);
        map.put("cnMontrayUnit",cnMontrayUnit);

        map.put("jobsList",mapList1);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;

    }


    @ApiOperation(value="获取推广员和保险代理人")
    @RequestMapping(value = "getPromoterAndAgent",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<PromoterAndAgentDto> getPromoterAndAgent(String applicationId,String policyId){

        MessageResult<PromoterAndAgentDto> msgResult = new MessageResult<>();

        PromoterAndAgentDto promoterAndAgentDto = new PromoterAndAgentDto();

        try {

            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            Company company = companyService.get(insuranceApplication.getCompanyId());

            String promoter = "";//推广员账号

            if (StringUtils.isNotEmpty(company.getPromoter())) {
                promoter = company.getPromoter();
            }

            List<JpCloudUser> promoterList = jpCloudUserService.findUserByNames(Arrays.asList(promoter));

            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            InsuranceAgent insuranceAgent = insurancePolicy.getAgent();

            if (promoterList.size()>0){
                JpCloudUser jpCloudUser = promoterList.get(0);
                //promoterAndAgentDto.setPromoter(jpCloudUser.getName() + "," + jpCloudUser.getRealName() + "," + jpCloudUser.getPhone());
                //sz
                promoterAndAgentDto.setPromoter(jpCloudUser.getName());
                promoterAndAgentDto.setPromoterName(jpCloudUser.getRealName());
                promoterAndAgentDto.setPromoterPhone(jpCloudUser.getPhone());
            }

            String agentName = "";

            if (insuranceAgent != null && StringUtils.isNotEmpty(insuranceAgent.getName())) {
                agentName = insuranceAgent.getName();
            }

            promoterAndAgentDto.setAgentName(agentName);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }


        msgResult.setResult(true);
        msgResult.setData(promoterAndAgentDto);

        return msgResult;
    }


}
