package com.jpsoft.picc.modules.business.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.business.entity.InsurancePolicyMember;
import com.jpsoft.picc.modules.business.service.InsurancePolicyMemberService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/insurancePolicyMember")
@Api(description = "每月投保单花名册")
public class InsurancePolicyMemberController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsurancePolicyMember> create(){
        MessageResult<InsurancePolicyMember> msgResult = new MessageResult<>();

        InsurancePolicyMember insurancePolicyMember = new InsurancePolicyMember();

        msgResult.setData(insurancePolicyMember);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsurancePolicyMember> add(@RequestBody InsurancePolicyMember insurancePolicyMember,@RequestAttribute String subject){
        MessageResult<InsurancePolicyMember> msgResult = new MessageResult<>();

        try {
            insurancePolicyMember.setId(UUID.randomUUID().toString());
            insurancePolicyMember.setDelFlag(false);
            insurancePolicyMember.setCreateBy(subject);
            insurancePolicyMember.setCreateTime(new Date());
            
            int affectCount = insurancePolicyMemberService.insert(insurancePolicyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsurancePolicyMember> edit(@PathVariable("id") String id){
        MessageResult<InsurancePolicyMember> msgResult = new MessageResult<>();

        try {
            InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(id);

            if (insurancePolicyMember != null) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsurancePolicyMember> update(@RequestBody InsurancePolicyMember insurancePolicyMember,@RequestAttribute String subject){
        MessageResult<InsurancePolicyMember> msgResult = new MessageResult<>();

        try {
            insurancePolicyMember.setUpdateBy(subject);
            insurancePolicyMember.setUpdateTime(new Date());
            
            int affectCount = insurancePolicyMemberService.update(insurancePolicyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicyMember);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(id);
            insurancePolicyMember.setDelFlag(true);
            insurancePolicyMember.setUpdateBy(subject);
            insurancePolicyMember.setUpdateTime(new Date());

            int affectCount = insurancePolicyMemberService.update(insurancePolicyMember);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(id);
                insurancePolicyMember.setDelFlag(true);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());

                affectCount += insurancePolicyMemberService.update(insurancePolicyMember);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }



    @ApiOperation(value="批量审批")
    @PostMapping("batchCheck")
    @ApiImplicitParams({
            @ApiImplicitParam(name="status",value = "状态",required = true,paramType = "query")
    })
    public MessageResult<Integer> batchCheck(@RequestBody List<String> idList,String status,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicyMember insurancePolicyMember = insurancePolicyMemberService.get(id);
                //状态改为初审
                insurancePolicyMember.setStatus(status);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());

                affectCount += insurancePolicyMemberService.update(insurancePolicyMember);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("审核失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> pageList(
            String policyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("status_","asc"));
        sortList.add(new Sort("create_time","asc"));

        if (StringUtils.isNotEmpty(policyId)) {
            searchParams.put("policyId",policyId);
        }

        Page<InsurancePolicyMember> page = insurancePolicyMemberService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
