package com.jpsoft.picc.modules.business.entity;

import java.io.Serializable;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.math.BigDecimal;

import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.InsuranceAgent;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;

/**
 描述:business_insurance_policy的实体类
 */
@Data
@ApiModel(value = "business_insurance_policy的实体类")
public class InsurancePolicy {
    @ApiModelProperty(value = "编号")
    private String id;
    @ApiModelProperty(value = "险种名称")
    private String definitionName;
    @ApiModelProperty(value = "保险种类")
    private String definitionId;
    @ApiModelProperty(value = "保险种类")
    private InsuranceDefinition definition;
    @ApiModelProperty(value = "状态:待制单50/待出单60/已出单70")
    private String status;
    @ApiModelProperty(value = "状态:草稿10/待初审20/待复审30/待缴费40/待制单50/待出单60/已出单70/已过期100(翻译)")
    private String statusN;
    @ApiModelProperty(value = "流程审核状态:正常/回退等")
    private String processStatus;
    @ApiModelProperty(value = "保险费合计(总金额)")
    private BigDecimal insuranceFee;

    @DateTimeFormat(pattern="yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd",timezone ="GMT+8")
    @ApiModelProperty(value = "开始时间")
    private Date startTime;

    @DateTimeFormat(pattern="yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd",timezone ="GMT+8")
    @ApiModelProperty(value = "结束时间")
    private Date endTime;
    @ApiModelProperty(value = "人数")
    private Integer number;
    @ApiModelProperty(value = "入账银行")
    private String accountBank;
    @ApiModelProperty(value = "出账银行")
    private String issuingBank;
    @ApiModelProperty(value = "单号")
    private String singlNumber;
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone ="GMT+8")
    @ApiModelProperty(value = "转账时间")
    private Date transferTime;
    @ApiModelProperty(value = "保险代理人编号")
    private String agentId;
    @ApiModelProperty(value = "保险代理人")
    private InsuranceAgent agent;
    @ApiModelProperty(value = "投保单附件")
    private String insurancePolicyFile;
    @ApiModelProperty(value = "当前第几份")
    private Integer no;
    @ApiModelProperty(value = "生效年月(yyyyMM)")
    private String effectiveDate;
    @ApiModelProperty(value = "是否删除")
    private Boolean delFlag;
    @ApiModelProperty(value = "创建人")
    private String createBy;

    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone ="GMT+8")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @ApiModelProperty(value = "修改人")
    private String updateBy;

    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone ="GMT+8")
    @ApiModelProperty(value = "修改时间")
    private Date updateTime;

    @ApiModelProperty(value = "投保企业编号")
    private String companyId;
    @ApiModelProperty(value = "投保企业")
    private Company company;
    @ApiModelProperty(value = "最后关联投保申请单编号")
    private String latestApplicationId;

    @ApiModelProperty(value = "投保单号")
    private String policyNo;

    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone ="GMT+8")
    @ApiModelProperty(value = "出单时间")
    private Date finishTime;

    @ApiModelProperty(value = "佣金金额")
    private BigDecimal commission;

    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone ="GMT+8")
    @ApiModelProperty(value = "佣金打款时间")
    private Date commissionTime;

    @ApiModelProperty(value = "保险代理人佣金")
    private BigDecimal agentFee;

    @ApiModelProperty(value = "社会推广员佣金")
    private BigDecimal promoterFee;

    @ApiModelProperty(value = "社会推广员用户名")
    private String promoter;

    /**
     * 状态翻译
     * @return
     */
    public String getStatusN() {
        String statusN = "";

        if(StringUtils.isNotEmpty(status)) {
            try {
                PolicyStatus policyStatus = PolicyStatus.valueOf(Integer.valueOf(status));

                if (policyStatus != null) {
                    statusN = policyStatus.getText();
                }
            }catch (Exception ex){
                System.out.println(ex.getMessage());
            }
        }

        return statusN;
    }
}