package com.jpsoft.picc.modules.business.service.impl;

import cn.hutool.core.date.DateTime;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.jpsoft.picc.modules.base.dao.InsuranceJobsDAO;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.business.dao.InsurancePolicyDAO;
import com.jpsoft.picc.modules.business.dao.InsurancePolicyMemberDAO;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.business.service.InsurancePolicyService;
import com.jpsoft.picc.modules.common.dto.Sort;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Transactional
@Component(value = "insurancePolicyService")
public class InsurancePolicyServiceImpl implements InsurancePolicyService {
    @Resource(name = "insurancePolicyDAO")
    private InsurancePolicyDAO insurancePolicyDAO;

    @Autowired
    private InsuranceJobsDAO insuranceJobsDAO;

    @Autowired
    private InsurancePolicyMemberDAO insurancePolicyMemberDAO;

    @Override
    public InsurancePolicy get(String id) {
        // TODO Auto-generated method stub
        return insurancePolicyDAO.get(id);
    }

    @Override
    public int insert(InsurancePolicy model) {
        // TODO Auto-generated method stub
        //model.setId(UUID.randomUUID().toString());

        return insurancePolicyDAO.insert(model);
    }

    @Override
    public int update(InsurancePolicy model) {
        // TODO Auto-generated method stub
        return insurancePolicyDAO.update(model);
    }

    @Override
    public int delete(String id) {
        // TODO Auto-generated method stub
        return insurancePolicyDAO.delete(id);
    }

    @Override
    public boolean exist(String id) {
        // TODO Auto-generated method stub
        int count = insurancePolicyDAO.exist(id);

        return count > 0 ? true : false;
    }

    @Override
    public List<InsurancePolicy> list() {
        // TODO Auto-generated method stub
        return insurancePolicyDAO.list();
    }

    @Override
    public List<InsurancePolicy> toBeAdjustedPolicy(String effectiveDate,String status){
        return insurancePolicyDAO.toBeAdjustedPolicy(effectiveDate,status);
    }

    @Override
    public Page<InsurancePolicy> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, List<Sort> sortList) {
        Page<InsurancePolicy> page = PageHelper.startPage(pageNumber, pageSize).doSelectPage(() -> {
            insurancePolicyDAO.search(searchParams, sortList);
        });

        return page;
    }

    @Override
    public boolean existByApplicationIdAndEffectiveDate(String applicationId, String effectiveDate) {
        long count = insurancePolicyDAO.countByApplicationIdAndEffectiveDate(applicationId,effectiveDate);

        return count>0;
    }

    @Override
    public Page<InsurancePolicy> nextMonthPageList(Map<String, Object> searchParams, int pageNumber, int pageSize, List<Sort> sortList) {
        Page<InsurancePolicy> page = PageHelper.startPage(pageNumber, pageSize).doSelectPage(() -> {
            insurancePolicyDAO.nextMonthPageList(searchParams, sortList);
        });

        return page;
    }

    @Override
    public List<InsurancePolicy> findNextMonthPolicy(String companyId, String definitionId,
                                                     Date startTime,Date endTime, String[] status) {
        return insurancePolicyDAO.findNextMonthPolicy(companyId,definitionId,startTime,endTime,status);
    }


    /**
     * 更新下月投保单的费用和人数
     * @param insurancePolicy
     * @param subject
     */
    public void updatePolicyNumAndAmount(InsurancePolicy insurancePolicy,String subject) {
        List<InsuranceJobs> InsuranceJobsList = insuranceJobsDAO.findByDefinitionId(insurancePolicy.getDefinitionId());

        int months = 1;

        BigDecimal totalAmount = new BigDecimal(0);
        long totalNumber = 0;

        for (InsuranceJobs insuranceJobs:InsuranceJobsList) {
            long count = insurancePolicyMemberDAO.countByPolicyIdAndJobsId(insurancePolicy.getId(),insuranceJobs.getJobsId());
            BigDecimal money = insuranceJobs.getCharges().multiply(new BigDecimal(months)).multiply(new BigDecimal(count));
            totalAmount = totalAmount.add(money);
            totalNumber += count;
        }

        //更新下月投保单的参保人数和参保费用
        insurancePolicy.setInsuranceFee(totalAmount);
        insurancePolicy.setNumber((int)totalNumber);

        insurancePolicy.setUpdateTime(new Date());
        insurancePolicy.setUpdateBy(subject);

        update(insurancePolicy);
    }

    @Override
    public Page<InsurancePolicy> myPageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, List<Sort> sortList) {
        Page<InsurancePolicy> page = PageHelper.startPage(pageNumber, pageSize).doSelectPage(() -> {
            insurancePolicyDAO.mySearch(searchParams, sortList);
        });

        return page;
    }

    @Override
    public Page<InsurancePolicy> hisPageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, List<Sort> sortList) {
        Page<InsurancePolicy> page = PageHelper.startPage(pageNumber, pageSize).doSelectPage(() -> {
            insurancePolicyDAO.hisSearch(searchParams, sortList);
        });

        return page;
    }
}