package com.jpsoft.picc.advice;

import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.HttpUtil;
import com.jpsoft.picc.modules.sys.entity.SysLog;
import com.jpsoft.picc.modules.sys.service.PermissionService;
import com.jpsoft.picc.modules.sys.service.SysLogService;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Date;

@Slf4j
@Aspect
@Component
public class PermissionAdvice {
    @Autowired
    private SysLogService sysLogService;

    @Pointcut("(execution(public * com.jpsoft.picc..controller.*.*(..)))")
    public void pointcut(){

    }

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        long start = System.currentTimeMillis();

        String classType = point.getTarget().getClass().getName();
        Class<?> clazz = Class.forName(classType);
        String clazzSimpleName = clazz.getSimpleName();
        MethodSignature methodSignature = (MethodSignature) point.getSignature();
        String methodName = methodSignature.getName();

        // 通过正则表达式判断当前url是否符合
        //PathMatcher matcher = new AntPathMatcher();

        StringBuilder pathBuilder = new StringBuilder();

        //查询类的RequestMapping注解
        RequestMapping classMapping = clazz.getAnnotation(RequestMapping.class);

        if(classMapping != null && classMapping.value().length>0){
            pathBuilder.append(classMapping.value()[0]);
        }

        //查询方法的RequestMapping注解
        Method method = methodSignature.getMethod();

        RequestMapping methodMapping = method.getAnnotation(RequestMapping.class);

        if(methodMapping != null && methodMapping.value().length>0){
            String subPath = methodMapping.value()[0];

            if(!subPath.startsWith("/")){
                pathBuilder.append("/");
            }

            pathBuilder.append(subPath);
        }

        ServletRequestAttributes requestAttrs = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        Object obj = null;

        SysLog sysLog = new SysLog();

        if(requestAttrs!=null) {
            HttpServletRequest request = requestAttrs.getRequest();

            sysLog.setRemoteIp(HttpUtil.getIpAddress(request));
            sysLog.setUrl(request.getRequestURI());

            log.warn("访问地址：" + request.getRequestURL().toString());
            log.warn("path=" + pathBuilder.toString());

            //1.这里获取到所有的参数值的数组
            Object[] args = point.getArgs();
            String[] parameterNames = methodSignature.getParameterNames();

            StringBuilder argBuilder = new StringBuilder();

            for (int i = 0;i<Math.min(args.length, parameterNames.length);i++) {
                if (argBuilder.length()!=0){
                    argBuilder.append("&");
                }

                if(args[i] != null) {
                    String value = args[i].toString();
                    value = value.length() > 100 ? value.substring(0, 100) : value;

                    if (parameterNames[i].equals("password")){
                        value = "******";
                    }

                    argBuilder.append(parameterNames[i] + "=" + value);
                }
            }

//            log.warn(argBuilder.toString());
            sysLog.setData(argBuilder.toString());

            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

            if(principal!=null) {
                String userId = principal.getName();
                sysLog.setUserId(userId);
            }

            sysLog.setCreateTime(new Date());

            // 执行切入方法
            obj = point.proceed();
        }

        long elapse = System.currentTimeMillis() - start;

        System.out.println(classType);
        log.warn(String.format("调用类%s方法%s耗时%s毫秒",clazzSimpleName,methodName,elapse));

        sysLog.setElapse(elapse);
        sysLog.setPointcut(clazzSimpleName + "->" + methodName);

        sysLogService.insert(sysLog);

        return obj;
    }
}
